from collections.abc import AsyncIterator, Iterator

from ..models.completions import Completion, CompletionDelta
from ..models.content import Content
from ..models.messages import AssistantMessage, Message


class SyncEngine:
    def complete(self, messages: list[Message], *, model: str) -> Completion:
        while True:
            completion: Completion
            for delta in self.stream(messages, model=model):
                completion = delta.full

            # TODO: Implement tool calls

            messages.append(AssistantMessage(content=completion.content))
            return completion

    def stream(
        self, messages: list[Message], *, model: str
    ) -> Iterator[CompletionDelta]:
        completion = Completion(content=[])
        for contents in self._stream_deltas(messages, model=model):
            # Append to the previous content if it's the same type
            if completion.content and isinstance(
                completion.content[-1], type(contents[0])
            ):
                # TODO: Work out why isinstance is not sufficient
                completion.content[-1].append(contents[0])  # type: ignore[arg-type]
                # Add any additional content items
                completion.content.extend(contents[1:])
            else:
                # Otherwise, append the new content
                completion.content.extend(contents)

            yield CompletionDelta(full=completion, delta=contents)

    def _stream_deltas(
        self, messages: list[Message], *, model: str
    ) -> Iterator[list[Content]]:
        raise NotImplementedError


class AsyncEngine:
    async def complete(self, messages: list[Message], *, model: str) -> Completion:
        while True:
            completion: Completion
            async for delta in self.stream(messages, model=model):
                completion = delta.full

            # TODO: Implement tool calls

            messages.append(AssistantMessage(content=completion.content))
            return completion

    async def stream(
        self, messages: list[Message], *, model: str
    ) -> AsyncIterator[CompletionDelta]:
        completion = Completion(content=[])
        async for contents in self._stream_deltas(messages, model=model):
            # Append to the previous content if it's the same type
            if completion.content and isinstance(
                completion.content[-1], type(contents[0])
            ):
                # TODO: Work out why isinstance is not sufficient
                completion.content[-1].append(contents[0])  # type: ignore[arg-type]
                # Add any additional content items
                completion.content.extend(contents[1:])
            else:
                # Otherwise, append the new content
                completion.content.extend(contents)

            yield CompletionDelta(full=completion, delta=contents)

    async def _stream_deltas(
        self, messages: list[Message], *, model: str
    ) -> AsyncIterator[list[Content]]:
        raise NotImplementedError
        yield []
