from collections.abc import AsyncIterator, Iterator

from .backends.engine import AsyncEngine, SyncEngine
from .models.completions import Completion, CompletionDelta
from .models.messages import AssistantMessage, Conversation, Message


class SyncCompleto:
    def __init__(self, default_engine: SyncEngine, default_model: str):
        self.default_engine = default_engine
        self.default_model = default_model

    def complete(
        self,
        conversation: Conversation,
        engine: SyncEngine | None = None,
        model: str | None = None,
    ) -> Completion:
        engine = engine or self.default_engine
        model = model or self.default_model

        return engine.complete(conversation.messages, model=model)

    def respond(
        self,
        conversation: Conversation,
        engine: SyncEngine | None = None,
        model: str | None = None,
    ) -> Message:
        engine = engine or self.default_engine
        model = model or self.default_model

        return AssistantMessage(
            content=self.complete(conversation, engine, model).content
        )

    def stream(
        self,
        conversation: Conversation,
        engine: SyncEngine | None = None,
        model: str | None = None,
    ) -> Iterator[CompletionDelta]:
        engine = engine or self.default_engine
        model = model or self.default_model

        yield from engine.stream(conversation.messages, model=model)


class AsyncCompleto:
    def __init__(self, default_engine: AsyncEngine, default_model: str):
        self.default_engine = default_engine
        self.default_model = default_model

    async def complete(
        self,
        conversation: Conversation,
        engine: AsyncEngine | None = None,
        model: str | None = None,
    ) -> Completion:
        engine = engine or self.default_engine
        model = model or self.default_model

        return await engine.complete(conversation.messages, model=model)

    async def stream(
        self,
        conversation: Conversation,
        engine: AsyncEngine | None = None,
        model: str | None = None,
    ) -> AsyncIterator[CompletionDelta]:
        engine = engine or self.default_engine
        model = model or self.default_model

        async for delta_completion in engine.stream(conversation.messages, model=model):
            yield delta_completion
